import os.path
try:
    # Python 2.5 includes pysqlite as sqlite3
    import sqlite3
except ImportError:
    from pysqlite2 import dbapi2 as sqlite3

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, data_dir
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from AccessControl.SimpleObjectPolicies import allow_type

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import permissions

from config import *

# Declare sqlite3 cursors and rows safe for untrusted code
allow_type(sqlite3.Cursor, 1)
allow_type(sqlite3.Row, 1)

class LinkTallyTool(UniqueObject, SimpleItem, PropertyManager,
                    ActionProviderBase):
    """Linktally support tool"""
    id = 'portal_linktally'
    meta_type = 'Linktally Tool'
    title = 'Manages link tallies'
    types = ()
    wfstates = ('published', 'visible')
    database = 'linktallies.db'

    _properties = (
        dict(id='types', type='lines', mode='w'),
        dict(id='wfstates', type='lines', mode='w'),
        dict(id='database', type='string', mode='w'),
    )
    
    manage_options = (PropertyManager.manage_options +
                      ActionProviderBase.manage_options +
                      SimpleItem.manage_options)

    security = ClassSecurityInfo()
    
    security.declareProtected(permissions.View, 'listTallyLinks')
    listTallyLinks = PageTemplateFile('templates/list_links.pt', GLOBALS)
    
    def _conn(self):
        conn = getattr(self, '_v_db_conn', None)
        if not conn:
            path = self.database
            if not os.path.isabs(path):
                path = os.path.abspath(os.path.join(data_dir, path))
            if not os.path.exists(path):
                return None
            # Connect with check_same_thread to False; Zope keeps threads 
            # neatly seperated for us and we should avoid the overhead.
            self._v_db_conn = conn = sqlite3.connect(path,
                                                     check_same_thread=False)
            conn.row_factory = sqlite3.Row
        return conn
    
    def _queryTopRank(self, period, resultsize=10, types=()):
        conn = self._conn()
        if not conn:
            return ()
        statement = 'select path, type from %stally ' % period
        if types:
            types = ', '.join(["'%s'" % (t.replace("'", "''")) for t in types])
            statement += 'where type in (%s) ' % types
        statement += 'order by count desc, path limit :resultsize'
        
	catalog = getToolByName(self, 'portal_catalog')
	portal_path = '/'.join(
            getToolByName(self, 'portal_url').getPortalObject().getPhysicalPath())

        top = []
        for row in conn.execute(statement, locals()):
            path = row['path'].encode('utf8')
            brains = catalog(path=dict(query=portal_path+path, depth=0))
            for brain in brains:
                if brain.getPath() == portal_path+path:
                    top.append(brain)
                    break
        return top

    security.declareProtected(permissions.View, 'listTopWeek')
    def listTopWeek(self, resultsize=10, types=()):
        """Returns a iterable listing the top paths in the past week

        Each result-row has two items, path and type, which can be accessed
        by index and by name (sequence and dict style)
        
        """
        return self._queryTopRank('week', resultsize, types)
    
    security.declareProtected(permissions.View, 'listTopMonth')
    def listTopMonth(self, resultsize=10, types=()):
        """Returns a iterable listing the top paths in the past month

        Each result-row has two items, path and type, which can be accessed
        by index and by name (sequence and dict style)
        
        """
        return self._queryTopRank('month', resultsize, types)
    
    security.declareProtected(permissions.View, 'listTopYear')
    def listTopYear(self, resultsize=10, types=()):
        """Returns a iterable listing the top paths in the past year

        Each result-row has two items, path and type, which can be accessed
        by index and by name (sequence and dict style)
        
        """
        return self._queryTopRank('year', resultsize, types)
    
InitializeClass(LinkTallyTool)
