import db, utils, tallies, query, logparser, config

def runLinkTally(conf):
    """Run a linktally based on conf settings"""
    assert isinstance(conf, config.LinkTallyConfig)

    # Retrieve a database connection, logfile and a linktallies instance
    conn = db.getConnection(conf)
    logfile = open(conf.logfile, 'rU') # Support universal line endings
    ltallies = tallies.LinkTallies(conn)
    
    # Query the CMS about links and types, collect the paths, and update
    # the linktallies database
    lquery = query.LinksQuery(conf.linksurl)
    collector = utils.pathcollector(lquery)
    ltallies.updatePaths(collector)
    paths = collector.paths
    
    # Retrieve the last parsed log position from the database
    offset = conn.getConfig('log-offset')
    checksum = conn.getConfig('log-checksum')
    timestamp = conn.getConfig('log-timestamp', type='timestamp')
    
    position = dict(offset=offset, checksum=checksum, timestamp=timestamp)
    
    # Build the log parse pipeline as a set of nested iterators
    parser = logparser.LogParser(logfile, position)
    # Remove query parts
    iterable = utils.removequery(parser)
    # Remove a prefix from logged paths, if defined
    if conf.prefix:
        iterable = utils.prefixfilter(iterable, conf.prefix)
        iterable = utils.removeprefix(iterable, conf.prefix)
    # Only tally paths we want to tally
    iterable = utils.pathfilter(iterable, paths)
    # We are not interested in the full datetime, only the date
    iterable = utils.todate(iterable)
    
    # Now we can count accesses, grouped by date
    for date, group in utils.groupby(iterable, lambda x: x[0]):
        counts = {}
        for date, path in group:
            counts[path] = counts.get(path, 0) + 1
        for path, count in counts.iteritems():
            ltallies.tallyPath(path, date, count)
    
    # Store the log position in the database
    position = parser.position
    for key, value in position.items():
        conn.setConfig('log-%s' % key, value)
        
    # Done!
