#!/usr/bin/python
# Copyright (C) 2006 Pareto; http://www.pareto.nl/
# Written by Martijn Pieters <martijn.pieters@pareto.nl>
# Licensed under the ZPL, see LICENSE.txt

'''usage: %prog [options] [configfile(s)]

Periodically parse a web-server access log and update link tallies for
specified links in a shared database. 

The links are specified by a CMS; by tallying links in a web-server log the CMS
avoids the expense of counting every link access themselves.

You must specify a config file, or specify all of --database, --linksurl and
--logfile in order to run. Command line options override any settings in a
config file. If you specify multiple configfiles, they are processed in order.

'''

VERSION = 'pre-0.1 alpha (unreleased)'
USAGE = __doc__.splitlines()[0]
DESCRIPTION = '\n'.join(__doc__.splitlines()[2:])

# Append local src dir if it exists so one doesn't *have* to install linktally
# in the pythonpath
import os, sys
srcdir = os.path.join(os.path.dirname(sys.argv[0]), 'src')
if os.path.exists(srcdir) and srcdir not in sys.path:
    sys.path.append(srcdir)

from linktally import runLinkTally

if __name__ == '__main__':
    import optparse, linktally.config
    parser = optparse.OptionParser(usage=USAGE, version='%%prog %s' % VERSION,
                                   description=DESCRIPTION)
    parser.add_option('-d', '--database', help='Use database DATABASE; this '
                      "will be created if it doesn't exist")
    parser.add_option('-u', '--linksurl', help='Retrieve links information '
                      'from LINKSURL')
    parser.add_option('-l', '--logfile', help='Count accesses from LOGFILE')
    parser.add_option('-p', '--prefix', help='Strip PREFIX from paths in the '
                      'log; only paths prefixed thus will be tallied')
    
    options, args = parser.parse_args()
    
    if not args:
        if not (options.database and options.linksurl and options.logfile):
            parser.error('Please specify a config file or a database, '
                         'linksurl and a logfile to parse.')
            
    configoptions = dict(database=options.database, linksurl=options.linksurl,
                         logfile=options.logfile)
    if options.prefix is not None:
        configoptions['prefix'] = options.prefix
    if args:
        for configfile in args:
            conf = linktally.config.LinkTallyConfig(configfile, 
                                                    **configoptions)
            runLinkTally(conf)
    else:
        runLinkTally(linktally.config.LinkTallyConfig(**configoptions))
